@@ -3,6 +3,7 @@ use crate::config::Mode;
 use crate::explorer;
 use crate::input::Key;
 use crate::lua;
+use crate::permissions::Permissions;
 use crate::ui::Layout;
 use anyhow::{bail, Result};
 use chrono::{DateTime, Local};
@@ -212,6 +213,7 @@ pub struct Node {
     pub mime_essence: String,
     pub size: u64,
     pub human_size: String,
+    pub permissions: Permissions,
     pub canonical: Option<ResolvedNode>,
     pub symlink: Option<ResolvedNode>,
 }
@@ -235,7 +237,7 @@ impl Node {
             .map(|p| (false, Some(ResolvedNode::from(p))))
             .unwrap_or_else(|_| (true, None));
 
-        let (is_symlink, is_dir, is_file, is_readonly, size) = path
+        let (is_symlink, is_dir, is_file, is_readonly, size, permissions) = path
             .symlink_metadata()
             .map(|m| {
                 (
@@ -244,9 +246,10 @@ impl Node {
                     m.is_file(),
                     m.permissions().readonly(),
                     m.len(),
+                    Permissions::from(&m),
                 )
             })
-            .unwrap_or((false, false, false, false, 0));
+            .unwrap_or_else(|_| (false, false, false, false, 0, Permissions::default()));
 
         let mime_essence = mime_guess::from_path(&path)
             .first()
@@ -268,6 +271,7 @@ impl Node {
             mime_essence,
             size,
             human_size,
+            permissions,
             canonical: maybe_canonical_meta.clone(),
             symlink: if is_symlink {
                 maybe_canonical_meta
@@ -346,6 +350,11 @@ impl Node {
     pub fn human_size(&self) -> &String {
         &self.human_size
     }
+
+    /// Get a reference to the node's permissions.
+    pub fn permissions(&self) -> &Permissions {
+        &self.permissions
+    }
 }
 
 impl Ord for Node {
@@ -1130,15 +1139,19 @@ pub enum ExternalMsg {
     /// **Example:** `CallSilently: {command: tput, args: ["bell"]}`
     CallSilently(Command),
 
-    /// Call a Lua function
+    /// Call a Lua function.
+    /// The complete app state (exportable using `PrintAppStateAndQuit` or `Debug`)
+    /// will be passed to the function as argument.
+    /// The function can optionally return a list of messages for xplr to handle
+    /// after the executing the function.
     ///
-    /// **Example:** `CallLua: custom.foo_funtion`
+    /// **Example:** `CallLua: custom.some_custom_funtion`
     CallLua(String),
 
     /// Like `CallLua` but without the flicker. The stdin, stdout
     /// stderr will be piped to null. So it's non-interactive.
     ///
-    /// **Example:** `CallLuaSilently: custom.bar_function`
+    /// **Example:** `CallLuaSilently: custom.some_custom_function`
     CallLuaSilently(String),
 
     /// An alias to `Call: {command: bash, args: ["-c", "${command}"], silent: false}`
@@ -1412,7 +1425,7 @@ impl std::fmt::Display for Log {
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub enum HelpMenuLine {
-    KeyMap(String, String),
+    KeyMap(String, Vec<String>, String),
     Paragraph(String),
 }
 
@@ -2615,16 +2628,8 @@ impl App {
                 .iter()
                 .map(|l| match l {
                     HelpMenuLine::Paragraph(p) => format!("\t{}\n", p),
-                    HelpMenuLine::KeyMap(k, h) => {
-                        let remaps = self
-                            .mode()
-                            .key_bindings()
-                            .remaps()
-                            .iter()
-                            .filter(|(_, maybeto)| maybeto.as_ref().map(|to| to == k).unwrap_or(false))
-                            .map(|(f, _)| f.clone())
-                            .collect::<Vec<String>>()
-                            .join(", ");
+                    HelpMenuLine::KeyMap(k, remaps, h) => {
+                        let remaps = remaps.join(", ");
                         format!(" {:15} | {:25} | {}\n", k, remaps, h)
                     }
                 })
