@@ -4,14 +4,14 @@ use crate::app;
 use crate::auto_refresher;
 use crate::event_reader;
 use crate::explorer;
+use crate::lua;
 use crate::pipe_reader;
 use crate::pwd_watcher;
 use crate::ui;
 use anyhow::Result;
+use crossterm::event;
 use crossterm::execute;
 use crossterm::terminal as term;
-use handlebars::{handlebars_helper, Handlebars};
-use humansize::{file_size_opts as options, FileSize};
 use std::fs;
 use std::io;
 use std::io::prelude::*;
@@ -21,7 +21,26 @@ use termion::get_tty;
 use tui::backend::CrosstermBackend;
 use tui::Terminal;
 
-handlebars_helper!(to_humansize: |size: i64| size.file_size(options::CONVENTIONAL).unwrap_or_default());
+fn call_lua(
+    app: &app::App,
+    lua: &mlua::Lua,
+    func: &str,
+    silent: bool,
+) -> Result<Option<Vec<app::ExternalMsg>>> {
+    let _focus_index = app
+        .directory_buffer()
+        .map(|d| d.focus())
+        .unwrap_or_default()
+        .to_string();
+
+    let (_, _, _) = if silent {
+        (Stdio::null(), Stdio::null(), Stdio::null())
+    } else {
+        (get_tty()?.into(), get_tty()?.into(), get_tty()?.into())
+    };
+
+    lua::call(lua, func, &app)
+}
 
 fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatus> {
     let focus_index = app
@@ -38,7 +57,6 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatu
 
     Command::new(cmd.command().clone())
         .env("XPLR_APP_VERSION", app.version())
-        .env("XPLR_CONFIG_VERSION", app.config().version())
         .env("XPLR_PID", &app.pid().to_string())
         .env("XPLR_INPUT_BUFFER", app.input_buffer().unwrap_or_default())
         .env("XPLR_FOCUS_PATH", app.focused_node_str())
@@ -65,11 +83,25 @@ fn call(app: &app::App, cmd: app::Command, silent: bool) -> io::Result<ExitStatu
         .status()
 }
 
-pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<String>> {
+pub fn run(
+    mut app: app::App,
+    focused_path: Option<String>,
+    lua: mlua::Lua,
+) -> Result<Option<String>> {
+    fs::create_dir_all(app.session_path())?;
+
     let (tx_msg_in, rx_msg_in) = mpsc::channel();
     let (tx_event_reader, rx_event_reader) = mpsc::channel();
     let (tx_pwd_watcher, rx_pwd_watcher) = mpsc::channel();
 
+    app = app.explore_pwd()?;
+
+    app = if let Some(f) = focused_path.clone() {
+        app.focus_by_file_name(&f, true)?
+    } else {
+        app.focus_first(true)?
+    };
+
     explorer::explore_recursive_async(
         app.explorer_config().clone(),
         app.pwd().clone(),
@@ -78,23 +110,6 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
     );
     tx_pwd_watcher.send(app.pwd().clone())?;
 
-    let mut hb = Handlebars::new();
-    hb.register_helper("humansize", Box::new(to_humansize));
-    hb.register_template_string(
-        app::TEMPLATE_TABLE_ROW,
-        &app.config()
-            .general()
-            .table()
-            .row()
-            .cols()
-            .clone()
-            .unwrap_or_default()
-            .iter()
-            .map(|c| c.format().clone().unwrap_or_default())
-            .collect::<Vec<String>>()
-            .join("\t"),
-    )?;
-
     let mut result = Ok(None);
     let session_path = app.session_path().to_owned();
 
@@ -102,16 +117,16 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
     let mut stdout = get_tty()?;
     // let mut stdout = stdout.lock();
     execute!(stdout, term::EnterAlternateScreen)?;
-    // let stdout = MouseTerminal::from(stdout);
+    execute!(stdout, event::EnableMouseCapture).unwrap_or_default(); // Optional
     let backend = CrosstermBackend::new(stdout);
     let mut terminal = Terminal::new(backend)?;
     terminal.hide_cursor()?;
 
     // Threads
     auto_refresher::start_auto_refreshing(tx_msg_in.clone());
-    pipe_reader::keep_reading(app.pipe().msg_in().clone(), tx_msg_in.clone());
     event_reader::keep_reading(tx_msg_in.clone(), rx_event_reader);
     pwd_watcher::keep_watching(app.pwd(), tx_msg_in.clone(), rx_pwd_watcher)?;
+    // pipe_reader::keep_reading(app.pipe().msg_in().clone(), tx_msg_in.clone());
 
     'outer: for task in rx_msg_in {
         match app.handle_task(task) {
@@ -149,23 +164,6 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                             terminal.clear()?;
                         }
 
-                        app::MsgOut::ExplorePwd => {
-                            match explorer::explore_sync(
-                                app.explorer_config().clone(),
-                                app.pwd().clone(),
-                                app.focused_node().map(|n| n.relative_path().clone()),
-                            ) {
-                                Ok(buf) => {
-                                    let pwd = buf.parent().clone();
-                                    app = app.add_directory(pwd.clone(), buf)?;
-                                }
-                                Err(e) => {
-                                    app = app.log_error(e.to_string())?;
-                                }
-                            };
-                            tx_pwd_watcher.send(app.pwd().clone())?;
-                        }
-
                         app::MsgOut::ExplorePwdAsync => {
                             explorer::explore_async(
                                 app.explorer_config().clone(),
@@ -190,14 +188,35 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                             // $PWD watcher
                             tx_pwd_watcher.send(app.pwd().clone())?;
                             // UI
-                            terminal.draw(|f| ui::draw(f, &app, &hb))?;
+                            terminal.draw(|f| ui::draw(f, &app, &lua))?;
+                        }
+
+                        app::MsgOut::CallLuaSilently(func) => {
+                            tx_event_reader.send(true)?;
+
+                            match call_lua(&app, &lua, &func, false) {
+                                Ok(Some(msgs)) => {
+                                    for msg in msgs {
+                                        app = app.handle_task(app::Task::new(
+                                            app::MsgIn::External(msg),
+                                            None,
+                                        ))?;
+                                    }
+                                }
+                                Ok(None) => {}
+                                Err(err) => {
+                                    app = app.log_error(err.to_string())?;
+                                }
+                            };
+
+                            tx_event_reader.send(false)?;
                         }
 
                         app::MsgOut::CallSilently(cmd) => {
                             tx_event_reader.send(true)?;
 
                             app.write_pipes()?;
-                            let status = call(&app, cmd, false)
+                            let status = call(&app, cmd, true)
                                 .map(|s| {
                                     if s.success() {
                                         Ok(())
@@ -207,6 +226,22 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                                 })
                                 .unwrap_or_else(|e| Err(e.to_string()));
 
+                            match pipe_reader::read_all(app.pipe().msg_in()) {
+                                Ok(msgs) => {
+                                    for msg in msgs {
+                                        app = app.handle_task(app::Task::new(
+                                            app::MsgIn::External(msg),
+                                            None,
+                                        ))?;
+                                    }
+                                }
+                                Err(err) => {
+                                    app = app.log_error(err.to_string())?;
+                                }
+                            };
+
+                            app.cleanup_pipes()?;
+
                             if let Err(e) = status {
                                 app = app.log_error(e.to_string())?;
                             };
@@ -214,7 +249,45 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                             tx_event_reader.send(false)?;
                         }
 
+                        app::MsgOut::CallLua(func) => {
+                            execute!(terminal.backend_mut(), event::DisableMouseCapture)
+                                .unwrap_or_default(); // Optional
+
+                            tx_event_reader.send(true)?;
+
+                            terminal.clear()?;
+                            terminal.set_cursor(0, 0)?;
+                            term::disable_raw_mode()?;
+                            terminal.show_cursor()?;
+
+                            match call_lua(&app, &lua, &func, false) {
+                                Ok(Some(msgs)) => {
+                                    for msg in msgs {
+                                        app = app.handle_task(app::Task::new(
+                                            app::MsgIn::External(msg),
+                                            None,
+                                        ))?;
+                                    }
+                                }
+                                Ok(None) => {}
+                                Err(err) => {
+                                    app = app.log_error(err.to_string())?;
+                                }
+                            };
+
+                            terminal.clear()?;
+                            term::enable_raw_mode()?;
+                            terminal.hide_cursor()?;
+                            tx_event_reader.send(false)?;
+
+                            execute!(terminal.backend_mut(), event::EnableMouseCapture)
+                                .unwrap_or_default(); // Optional
+                        }
+
                         app::MsgOut::Call(cmd) => {
+                            execute!(terminal.backend_mut(), event::DisableMouseCapture)
+                                .unwrap_or_default(); // Optional
+
                             tx_event_reader.send(true)?;
 
                             terminal.clear()?;
@@ -233,6 +306,22 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                                 })
                                 .unwrap_or_else(|e| Err(e.to_string()));
 
+                            match pipe_reader::read_all(app.pipe().msg_in()) {
+                                Ok(msgs) => {
+                                    for msg in msgs {
+                                        app = app.handle_task(app::Task::new(
+                                            app::MsgIn::External(msg),
+                                            None,
+                                        ))?;
+                                    }
+                                }
+                                Err(err) => {
+                                    app = app.log_error(err.to_string())?;
+                                }
+                            };
+
+                            app.cleanup_pipes()?;
+
                             if let Err(e) = status {
                                 app = app.log_error(e.to_string())?;
                             };
@@ -241,6 +330,9 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
                             term::enable_raw_mode()?;
                             terminal.hide_cursor()?;
                             tx_event_reader.send(false)?;
+
+                            execute!(terminal.backend_mut(), event::EnableMouseCapture)
+                                .unwrap_or_default(); // Optional
                         }
                     };
                 }
@@ -256,10 +348,11 @@ pub fn run(mut app: app::App, focused_path: Option<String>) -> Result<Option<Str
     terminal.clear()?;
     terminal.set_cursor(0, 0)?;
     execute!(terminal.backend_mut(), term::LeaveAlternateScreen)?;
+    execute!(terminal.backend_mut(), event::DisableMouseCapture).unwrap_or_default(); // Optional
     term::disable_raw_mode()?;
     terminal.show_cursor()?;
 
-    fs::remove_dir_all(session_path)?;
+    fs::remove_dir(session_path)?;
 
     result
 }
