@@ -3,6 +3,9 @@ use crate::app::HelpMenuLine;
 use crate::app::{Node, ResolvedNode};
 use crate::config::PanelUiConfig;
 use crate::lua;
+use crate::permissions::Permissions;
+use ansi_to_tui::ansi_to_text;
+use anyhow::Result;
 use indexmap::IndexSet;
 use lazy_static::lazy_static;
 use mlua::Lua;
@@ -15,7 +18,7 @@ use tui::backend::Backend;
 use tui::layout::Rect;
 use tui::layout::{Constraint as TuiConstraint, Direction, Layout as TuiLayout};
 use tui::style::{Color, Modifier as TuiModifier, Style as TuiStyle};
-use tui::text::{Span, Spans};
+use tui::text::{Span, Spans, Text};
 use tui::widgets::{Block, Borders as TuiBorders, Cell, List, ListItem, Paragraph, Row, Table};
 use tui::Frame;
 
@@ -24,6 +27,14 @@ lazy_static! {
     pub static ref DEFAULT_STYLE: TuiStyle = TuiStyle::default();
 }
 
+fn read_only_indicator(app: &app::App) -> &str {
+    if app.config().general().read_only().unwrap_or_default() {
+        "(r)"
+    } else {
+        ""
+    }
+}
+
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct LayoutOptions {
@@ -319,7 +330,7 @@ impl From<ResolvedNode> for ResolvedNodeUiMetadata {
 }
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
-struct NodeUiMetadata {
+pub struct NodeUiMetadata {
     // From Node
     pub parent: String,
     pub relative_path: String,
@@ -333,6 +344,7 @@ struct NodeUiMetadata {
     pub mime_essence: String,
     pub size: u64,
     pub human_size: String,
+    pub permissions: Permissions,
     pub canonical: Option<ResolvedNodeUiMetadata>,
     pub symlink: Option<ResolvedNodeUiMetadata>,
 
@@ -378,6 +390,7 @@ impl NodeUiMetadata {
             mime_essence: node.mime_essence().clone(),
             size: node.size(),
             human_size: node.human_size().clone(),
+            permissions: node.permissions().to_owned(),
             canonical: node.canonical().to_owned().map(|s| s.into()),
             symlink: node.symlink().to_owned().map(|s| s.into()),
             index,
@@ -462,6 +475,10 @@ fn draw_table<B: Backend>(
                         })
                         .unwrap_or_default();
 
+                    let mut me = node.mime_essence().splitn(2, '/');
+                    let mimetype: String = me.next().map(|s| s.into()).unwrap_or_default();
+                    let mimesub: String = me.next().map(|s| s.into()).unwrap_or_default();
+
                     let node_type = app_config
                         .node_types()
                         .special()
@@ -471,7 +488,8 @@ fn draw_table<B: Backend>(
                             app_config
                                 .node_types()
                                 .mime_essence()
-                                .get(node.mime_essence())
+                                .get(&mimetype)
+                                .and_then(|t| t.get(&mimesub).or_else(|| t.get("*")))
                         })
                         .unwrap_or_else(|| {
                             if node.is_symlink() {
@@ -541,10 +559,15 @@ fn draw_table<B: Backend>(
                                 .iter()
                                 .filter_map(|c| {
                                     c.format().as_ref().map(|f| {
-                                        lua::call(lua, f, &v).unwrap_or_else(|e| e.to_string())
+                                        let out: Result<String> = lua::call(lua, f, &v);
+                                        match out {
+                                            Ok(o) => ansi_to_text(o.bytes())
+                                                .unwrap_or_else(|e| Text::raw(format!("{:?}", e))),
+                                            Err(e) => Text::raw(e.to_string()),
+                                        }
                                     })
                                 })
-                                .collect::<Vec<String>>()
+                                .collect::<Vec<Text>>()
                         })
                         .unwrap_or_default()
                         .iter()
@@ -658,31 +681,20 @@ fn draw_help_menu<B: Backend>(
         .into_iter()
         .map(|l| match l {
             HelpMenuLine::Paragraph(p) => Row::new([Cell::from(p)].to_vec()),
-            HelpMenuLine::KeyMap(k, h) => {
-                let remaps = app
-                    .mode()
-                    .key_bindings()
-                    .remaps()
-                    .iter()
-                    .filter(|(_, maybeto)| maybeto.as_ref().map(|to| to == &k).unwrap_or(false))
-                    .map(|(f, _)| f.clone())
-                    .collect::<Vec<String>>()
-                    .join("|");
-                Row::new([Cell::from(k), Cell::from(remaps), Cell::from(h)].to_vec())
+            HelpMenuLine::KeyMap(k, remaps, h) => {
+                Row::new([Cell::from(k), Cell::from(remaps.join("|")), Cell::from(h)].to_vec())
             }
         })
         .collect::<Vec<Row>>();
 
-    let read_only_indicator = if app.config().general().read_only().unwrap_or_default() {
-        "(r)"
-    } else {
-        ""
-    };
-
     let help_menu = Table::new(help_menu_rows)
         .block(block(
             config,
-            format!(" Help [{}{}] ", &app.mode().name(), read_only_indicator),
+            format!(
+                " Help [{}{}] ",
+                &app.mode().name(),
+                read_only_indicator(app)
+            ),
         ))
         .widths(&[
             TuiConstraint::Percentage(20),
@@ -725,7 +737,14 @@ fn draw_input_buffer<B: Backend>(
             app.config().general().cursor().style().clone().into(),
         ),
     ]))
-    .block(block(config, " Input ".into()));
+    .block(block(
+        config,
+        format!(
+            " Input [{}{}] ",
+            app.mode().name(),
+            read_only_indicator(app)
+        ),
+    ));
     f.render_widget(input_buf, layout_size);
 }
 
@@ -874,7 +893,15 @@ fn draw_logs<B: Backend>(
             .collect::<Vec<ListItem>>()
     };
 
-    let logs_list = List::new(logs).block(block(config, format!(" Logs ({}) ", app.logs().len())));
+    let logs_list = List::new(logs).block(block(
+        config,
+        format!(
+            " Logs ({}) [{}{}] ",
+            app.logs().len(),
+            app.mode().name(),
+            read_only_indicator(app),
+        ),
+    ));
 
     f.render_widget(logs_list, layout_size);
 }
